@@ -126,6 +126,7 @@ class TestCinderAllCmd(test.TestCase):
     @mock.patch('oslo_log.log.setup')
     def test_main(self, log_setup, get_logger, monkey_patch, process_launcher,
                   wsgi_service, service_create, rpc_init):
+        CONF.set_override('enabled_backends', None)
         launcher = process_launcher.return_value
         server = wsgi_service.return_value
         server.workers = mock.sentinel.worker_count
@@ -152,6 +153,46 @@ class TestCinderAllCmd(test.TestCase):
 
         launcher.wait.assert_called_once_with()
 
+    @mock.patch('cinder.rpc.init')
+    @mock.patch('cinder.service.Service.create')
+    @mock.patch('cinder.service.WSGIService')
+    @mock.patch('cinder.service.process_launcher')
+    @mock.patch('cinder.utils.monkey_patch')
+    @mock.patch('oslo_log.log.getLogger')
+    @mock.patch('oslo_log.log.setup')
+    def test_main_with_backend(self, log_setup, get_logger, monkey_patch,
+                               process_launcher, wsgi_service, service_create,
+                               rpc_init):
+        CONF.set_override('enabled_backends', ['backend1'])
+        CONF.set_override('host', 'host')
+        launcher = process_launcher.return_value
+        server = wsgi_service.return_value
+        server.workers = mock.sentinel.worker_count
+        service = service_create.return_value
+
+        cinder_all.main()
+
+        self.assertEqual('cinder', CONF.project)
+        self.assertEqual(CONF.version, version.version_string())
+        log_setup.assert_called_once_with(CONF, "cinder")
+        get_logger.assert_called_once_with('cinder.all')
+        monkey_patch.assert_called_once_with()
+        rpc_init.assert_called_once_with(CONF)
+        process_launcher.assert_called_once_with()
+        wsgi_service.assert_called_once_with('osapi_volume')
+        launcher.launch_service.assert_any_call(server, workers=server.workers)
+
+        service_create.assert_has_calls([mock.call(binary='cinder-scheduler'),
+                                         mock.call(binary='cinder-backup'),
+                                         mock.call(binary='cinder-volume',
+                                                   host='host@backend1',
+                                                   service_name='backend1')])
+        self.assertEqual(3, service_create.call_count)
+        launcher.launch_service.assert_has_calls([mock.call(service)] * 3)
+        self.assertEqual(4, launcher.launch_service.call_count)
+
+        launcher.wait.assert_called_once_with()
+
     @mock.patch('cinder.rpc.init')
     @mock.patch('cinder.service.Service.create')
     @mock.patch('cinder.service.WSGIService')
@@ -204,6 +245,7 @@ class TestCinderAllCmd(test.TestCase):
                                         monkey_patch, process_launcher,
                                         wsgi_service, service_create,
                                         rpc_init):
+        CONF.set_override('enabled_backends', None)
         launcher = process_launcher.return_value
         server = wsgi_service.return_value
         server.workers = mock.sentinel.worker_count
@@ -774,6 +816,9 @@ class TestCinderRtstoolCmd(test.TestCase):
         sys.argv = ['cinder-rtstool']
         CONF(sys.argv[1:], project='cinder', version=version.version_string())
 
+        self.INITIATOR_IQN = 'iqn.2015.12.com.example.openstack.i:UNIT1'
+        self.TARGET_IQN = 'iqn.2015.12.com.example.openstack.i:TARGET1'
+
     def tearDown(self):
         super(TestCinderRtstoolCmd, self).tearDown()
 
@@ -944,7 +989,7 @@ class TestCinderRtstoolCmd(test.TestCase):
             self.assertRaises(rtslib_fb.utils.RTSLibError,
                               cinder_rtstool.add_initiator,
                               mock.sentinel.target_iqn,
-                              mock.sentinel.initiator_iqn,
+                              self.INITIATOR_IQN,
                               mock.sentinel.userid,
                               mock.sentinel.password)
 
@@ -955,7 +1000,7 @@ class TestCinderRtstoolCmd(test.TestCase):
         self.assertRaises(cinder_rtstool.RtstoolError,
                           cinder_rtstool.add_initiator,
                           mock.sentinel.target_iqn,
-                          mock.sentinel.initiator_iqn,
+                          self.INITIATOR_IQN,
                           mock.sentinel.userid,
                           mock.sentinel.password)
 
@@ -965,15 +1010,64 @@ class TestCinderRtstoolCmd(test.TestCase):
     def test_add_initiator_acl_exists(self, rtsroot, node_acl, mapped_lun):
         target_iqn = mock.MagicMock()
         target_iqn.tpgs.return_value = \
-            [{'node_acls': mock.sentinel.initiator_iqn}]
-        acl = mock.MagicMock(node_wwn=mock.sentinel.initiator_iqn)
+            [{'node_acls': self.INITIATOR_IQN}]
+        acl = mock.MagicMock(node_wwn=self.INITIATOR_IQN)
+        tpg = mock.MagicMock(node_acls=[acl])
+        tpgs = iter([tpg])
+        target = mock.MagicMock(tpgs=tpgs, wwn=self.TARGET_IQN)
+        rtsroot.return_value = mock.MagicMock(targets=[target])
+
+        cinder_rtstool.add_initiator(self.TARGET_IQN,
+                                     self.INITIATOR_IQN,
+                                     mock.sentinel.userid,
+                                     mock.sentinel.password)
+        self.assertFalse(node_acl.called)
+        self.assertFalse(mapped_lun.called)
+
+    @mock.patch.object(rtslib_fb, 'MappedLUN')
+    @mock.patch.object(rtslib_fb, 'NodeACL')
+    @mock.patch.object(rtslib_fb.root, 'RTSRoot')
+    def test_add_initiator_acl_exists_case_1(self,
+                                             rtsroot,
+                                             node_acl,
+                                             mapped_lun):
+        """Ensure initiator iqns are handled in a case-insensitive manner."""
+        target_iqn = mock.MagicMock()
+        target_iqn.tpgs.return_value = \
+            [{'node_acls': self.INITIATOR_IQN.lower()}]
+        acl = mock.MagicMock(node_wwn=self.INITIATOR_IQN)
         tpg = mock.MagicMock(node_acls=[acl])
         tpgs = iter([tpg])
         target = mock.MagicMock(tpgs=tpgs, wwn=target_iqn)
         rtsroot.return_value = mock.MagicMock(targets=[target])
 
         cinder_rtstool.add_initiator(target_iqn,
-                                     mock.sentinel.initiator_iqn,
+                                     self.INITIATOR_IQN,
+                                     mock.sentinel.userid,
+                                     mock.sentinel.password)
+        self.assertFalse(node_acl.called)
+        self.assertFalse(mapped_lun.called)
+
+    @mock.patch.object(rtslib_fb, 'MappedLUN')
+    @mock.patch.object(rtslib_fb, 'NodeACL')
+    @mock.patch.object(rtslib_fb.root, 'RTSRoot')
+    def test_add_initiator_acl_exists_case_2(self,
+                                             rtsroot,
+                                             node_acl,
+                                             mapped_lun):
+        """Ensure initiator iqns are handled in a case-insensitive manner."""
+        iqn_lower = self.INITIATOR_IQN.lower()
+        target_iqn = mock.MagicMock()
+        target_iqn.tpgs.return_value = \
+            [{'node_acls': self.INITIATOR_IQN}]
+        acl = mock.MagicMock(node_wwn=iqn_lower)
+        tpg = mock.MagicMock(node_acls=[acl])
+        tpgs = iter([tpg])
+        target = mock.MagicMock(tpgs=tpgs, wwn=target_iqn)
+        rtsroot.return_value = mock.MagicMock(targets=[target])
+
+        cinder_rtstool.add_initiator(target_iqn,
+                                     self.INITIATOR_IQN,
                                      mock.sentinel.userid,
                                      mock.sentinel.password)
         self.assertFalse(node_acl.called)
@@ -985,7 +1079,7 @@ class TestCinderRtstoolCmd(test.TestCase):
     def test_add_initiator(self, rtsroot, node_acl, mapped_lun):
         target_iqn = mock.MagicMock()
         target_iqn.tpgs.return_value = \
-            [{'node_acls': mock.sentinel.initiator_iqn}]
+            [{'node_acls': self.INITIATOR_IQN}]
         tpg = mock.MagicMock()
         tpgs = iter([tpg])
         target = mock.MagicMock(tpgs=tpgs, wwn=target_iqn)
@@ -996,11 +1090,11 @@ class TestCinderRtstoolCmd(test.TestCase):
         node_acl.return_value = acl_new
 
         cinder_rtstool.add_initiator(target_iqn,
-                                     mock.sentinel.initiator_iqn,
+                                     self.INITIATOR_IQN,
                                      mock.sentinel.userid,
                                      mock.sentinel.password)
         node_acl.assert_called_once_with(tpg,
-                                         mock.sentinel.initiator_iqn,
+                                         self.INITIATOR_IQN,
                                          mode='create')
         mapped_lun.assert_called_once_with(acl_new, 0, tpg_lun=0)
 
@@ -1029,6 +1123,40 @@ class TestCinderRtstoolCmd(test.TestCase):
         target.delete.assert_called_once_with()
         storage_object.delete.assert_called_once_with()
 
+    @mock.patch.object(rtslib_fb, 'MappedLUN')
+    @mock.patch.object(rtslib_fb, 'NodeACL')
+    @mock.patch.object(rtslib_fb.root, 'RTSRoot')
+    def test_delete_initiator(self, rtsroot, node_acl, mapped_lun):
+        target_iqn = mock.MagicMock()
+        target_iqn.tpgs.return_value = \
+            [{'node_acls': self.INITIATOR_IQN}]
+        acl = mock.MagicMock(node_wwn=self.INITIATOR_IQN)
+        tpg = mock.MagicMock(node_acls=[acl])
+        tpgs = iter([tpg])
+        target = mock.MagicMock(tpgs=tpgs, wwn=target_iqn)
+        rtsroot.return_value = mock.MagicMock(targets=[target])
+
+        cinder_rtstool.delete_initiator(target_iqn,
+                                        self.INITIATOR_IQN)
+
+    @mock.patch.object(rtslib_fb, 'MappedLUN')
+    @mock.patch.object(rtslib_fb, 'NodeACL')
+    @mock.patch.object(rtslib_fb.root, 'RTSRoot')
+    def test_delete_initiator_case(self, rtsroot, node_acl, mapped_lun):
+        """Ensure iqns are handled in a case-insensitive manner."""
+        initiator_iqn_lower = self.INITIATOR_IQN.lower()
+        target_iqn = mock.MagicMock()
+        target_iqn.tpgs.return_value = \
+            [{'node_acls': initiator_iqn_lower}]
+        acl = mock.MagicMock(node_wwn=self.INITIATOR_IQN)
+        tpg = mock.MagicMock(node_acls=[acl])
+        tpgs = iter([tpg])
+        target = mock.MagicMock(tpgs=tpgs, wwn=target_iqn)
+        rtsroot.return_value = mock.MagicMock(targets=[target])
+
+        cinder_rtstool.delete_initiator(target_iqn,
+                                        self.INITIATOR_IQN)
+
     @mock.patch.object(cinder_rtstool, 'os', autospec=True)
     @mock.patch.object(cinder_rtstool, 'rtslib_fb', autospec=True)
     def test_save_with_filename(self, mock_rtslib, mock_os):
