@@ -82,7 +82,7 @@ STORAGE_POOL_NAME = 'sio:sp_name'
 STORAGE_POOL_ID = 'sio:sp_id'
 PROTECTION_DOMAIN_NAME = 'sio:pd_name'
 PROTECTION_DOMAIN_ID = 'sio:pd_id'
-PROVISIONING_KEY = 'sio:provisioning'
+PROVISIONING_KEY = 'sio:provisioning_type'
 IOPS_LIMIT_KEY = 'sio:iops_limit'
 BANDWIDTH_LIMIT = 'sio:bandwidth_limit'
 
@@ -122,8 +122,13 @@ class ScaleIODriver(driver.VolumeDriver):
              'user': self.server_username,
              'verify_cert': self.verify_server_certificate})
 
-        self.storage_pools = [e.strip() for e in
-                              self.configuration.sio_storage_pools.split(',')]
+        self.storage_pools = None
+        if self.configuration.sio_storage_pools:
+            self.storage_pools = [
+                e.strip() for e in
+                self.configuration.sio_storage_pools.split(',')
+            ]
+
         self.storage_pool_name = self.configuration.sio_storage_pool_name
         self.storage_pool_id = self.configuration.sio_storage_pool_id
         if self.storage_pool_name is None and self.storage_pool_id is None:
@@ -147,9 +152,7 @@ class ScaleIODriver(driver.VolumeDriver):
             {'domain_id': self.protection_domain_id})
 
         self.connector = connector.InitiatorConnector.factory(
-            # TODO(xyang): Change 'SCALEIO' to connector.SCALEIO after
-            # os-brick 0.4.0 is released.
-            'SCALEIO', utils.get_root_helper(),
+            connector.SCALEIO, utils.get_root_helper(),
             device_scan_attempts=
             self.configuration.num_volume_device_scan_tries
         )
@@ -205,6 +208,12 @@ class ScaleIODriver(driver.VolumeDriver):
             msg = _("Must specify storage pool name or id.")
             raise exception.InvalidInput(reason=msg)
 
+        if not self.storage_pools:
+            msg = _(
+                "Must specify storage pools. Option: sio_storage_pools."
+            )
+            raise exception.InvalidInput(reason=msg)
+
     def _find_storage_pool_id_from_storage_type(self, storage_type):
         # Default to what was configured in configuration file if not defined.
         return storage_type.get(STORAGE_POOL_ID,
@@ -512,10 +521,11 @@ class ScaleIODriver(driver.VolumeDriver):
         return verify_cert
 
     def extend_volume(self, volume, new_size):
-        """Extends the size of an existing available ScaleIO volume."""
-
-        self._check_volume_size(new_size)
+        """Extends the size of an existing available ScaleIO volume.
 
+        This action will round up the volume to the nearest size that is
+        a granularity of 8 GBs.
+        """
         vol_id = volume['provider_id']
         LOG.info(_LI(
             "ScaleIO extend volume: volume %(volname)s to size %(new_size)s."),
@@ -529,7 +539,20 @@ class ScaleIODriver(driver.VolumeDriver):
                    "/api/instances/Volume::%(vol_id)s"
                    "/action/setVolumeSize") % req_vars
         LOG.info(_LI("Change volume capacity request: %s."), request)
-        volume_new_size = new_size
+
+        # Round up the volume size so that it is a granularity of 8 GBs
+        # because ScaleIO only supports volumes with a granularity of 8 GBs.
+        if new_size % 8 == 0:
+            volume_new_size = new_size
+        else:
+            volume_new_size = new_size + 8 - (new_size % 8)
+
+        round_volume_capacity = self.configuration.sio_round_volume_capacity
+        if (not round_volume_capacity and not new_size % 8 == 0):
+            LOG.warning(_LW("ScaleIO only supports volumes with a granularity "
+                            "of 8 GBs. The new volume size is: %d."),
+                        volume_new_size)
+
         params = {'sizeInGB': six.text_type(volume_new_size)}
         r = requests.post(
             request,
@@ -896,6 +919,75 @@ class ScaleIODriver(driver.VolumeDriver):
         finally:
             self._sio_detach_volume(volume)
 
+    def update_migrated_volume(self, ctxt, volume, new_volume,
+                               original_volume_status):
+        """Return the update from ScaleIO migrated volume.
+
+        This method updates the volume name of the new ScaleIO volume to
+        match the updated volume ID.
+        The original volume is renamed first since ScaleIO does not allow
+        multiple volumes to have the same name.
+        """
+        name_id = None
+        location = None
+        if original_volume_status == 'available':
+            # During migration, a new volume is created and will replace
+            # the original volume at the end of the migration. We need to
+            # rename the new volume. The current_name of the new volume,
+            # which is the id of the new volume, will be changed to the
+            # new_name, which is the id of the original volume.
+            current_name = new_volume['id']
+            new_name = volume['id']
+            vol_id = new_volume['provider_id']
+            LOG.info(_LI("Renaming %(id)s from %(current_name)s to "
+                         "%(new_name)s."),
+                     {'id': vol_id, 'current_name': current_name,
+                      'new_name': new_name})
+
+            # Original volume needs to be renamed first
+            self._rename_volume(volume, "ff" + new_name)
+            self._rename_volume(new_volume, new_name)
+        else:
+            # The back-end will not be renamed.
+            name_id = new_volume['_name_id'] or new_volume['id']
+            location = new_volume['provider_location']
+
+        return {'_name_id': name_id, 'provider_location': location}
+
+    def _rename_volume(self, volume, new_id):
+        new_name = self._id_to_base64(new_id)
+        vol_id = volume['provider_id']
+
+        req_vars = {'server_ip': self.server_ip,
+                    'server_port': self.server_port,
+                    'id': vol_id}
+        request = ("https://%(server_ip)s:%(server_port)s"
+                   "/api/instances/Volume::%(id)s/action/setVolumeName" %
+                   req_vars)
+        LOG.info(_LI("ScaleIO rename volume request: %s."), request)
+
+        params = {'newName': new_name}
+        r = requests.post(
+            request,
+            data=json.dumps(params),
+            headers=self._get_headers(),
+            auth=(self.server_username,
+                  self.server_token),
+            verify=self._get_verify_cert()
+        )
+        r = self._check_response(r, request, False, params)
+
+        if r.status_code != OK_STATUS_CODE:
+            response = r.json()
+            msg = (_("Error renaming volume %(vol)s: %(err)s.") %
+                   {'vol': vol_id, 'err': response['message']})
+            LOG.error(msg)
+            raise exception.VolumeBackendAPIException(data=msg)
+        else:
+            LOG.info(_LI("ScaleIO volume %(vol)s was renamed to "
+                         "%(new_name)s."),
+                     {'vol': vol_id, 'new_name': new_name})
+
     def ensure_export(self, context, volume):
         """Driver entry point to get the export info for an existing volume."""
         pass
