@@ -428,6 +428,26 @@ class NetAppESeriesLibrary(object):
         except exception.NetAppDriverException as e:
             with excutils.save_and_reraise_exception():
                 LOG.error(_LE("Error creating volume. Msg - %s."), e)
+                # There was some kind failure creating the volume, make sure no
+                # partial flawed work exists
+                try:
+                    bad_vol = self._get_volume(eseries_volume_label)
+                except Exception:
+                    # Swallowing the exception intentionally because this is
+                    # emergency cleanup to make sure no intermediate volumes
+                    # were left. In this whole error situation, the more
+                    # common route would be for no volume to have been created.
+                    pass
+                else:
+                    # Some sort of partial volume was created despite the
+                    # error.  Lets clean it out so no partial state volumes or
+                    # orphans are left.
+                    try:
+                        self._client.delete_volume(bad_vol["id"])
+                    except exception.NetAppDriverException as e2:
+                        LOG.error(_LE(
+                            "Error cleaning up failed volume creation.  "
+                            "Msg - %s."), e2)
 
         return vol
 
